@@ -1,4 +1,4 @@
-#!/usr/bin/python -u
+#!/usr/bin/env python
 #
 # generate python wrappers from the XML API description
 #
@@ -465,6 +465,7 @@ skip_impl = (
     'virDomainGetBlockJobInfo',
     'virDomainMigrateGetCompressionCache',
     'virDomainMigrateGetMaxSpeed',
+    'virDomainMigrateGetMaxDowntime',
     'virDomainBlockStatsFlags',
     'virDomainSetBlockIoTune',
     'virDomainGetBlockIoTune',
@@ -543,6 +544,11 @@ skip_function = (
     'virStreamSendAll', # Pure python libvirt-override-virStream.py
     'virStreamRecv', # overridden in libvirt-override-virStream.py
     'virStreamSend', # overridden in libvirt-override-virStream.py
+    'virStreamRecvHole', # overridden in libvirt-override-virStream.py
+    'virStreamSendHole', # overridden in libvirt-override-virStream.py
+    'virStreamRecvFlags', # overridden in libvirt-override-virStream.py
+    'virStreamSparseRecvAll', # overridden in libvirt-override-virStream.py
+    'virStreamSparseSendAll', # overridden in libvirt-override-virStream.py
 
     'virConnectUnregisterCloseCallback', # overridden in virConnect.py
     'virConnectRegisterCloseCallback', # overridden in virConnect.py
