@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 import sys
 import lxml
@@ -351,7 +351,8 @@ for klass in gotfunctions:
     for func in sorted(gotfunctions[klass]):
         # These are pure python methods with no C APi
         if func in ["connect", "getConnect", "domain", "getDomain",
-                    "virEventInvokeFreeCallback"]:
+                    "virEventInvokeFreeCallback",
+                    "sparseRecvAll", "sparseSendAll"]:
             continue
 
         key = "%s.%s" % (klass, func)
